module CabezaDeTermo::AlertMessages
	class Messages
		ALERT_MESSAGES = :alert_messages

		# Instance methods

		def initialize(flash:)
			@messages = Hash.new { |hash, key| hash[key] = [] }
			@flash = flash
			transfer_flash_messages_for_this_request
		end

		def [](message_type)
			messages[message_type]
		end

		def flash_at(message_type)
			ensure_flash_message_type_exists message_type
			flash_messages[message_type]
		end

		protected

		# Accessing flash oh oh

		def flash_messages()
			@flash[ALERT_MESSAGES]
		end

		def transfer_flash_messages_for_this_request()
			return if flash_messages.nil?

			flash_messages.each_pair do |type, messages|
				self[type].concat messages
			end

			drop_flash_messages
		end

		def drop_flash_messages()
			@flash[ALERT_MESSAGES] = nil
		end

		def reset_flash_messages()
			@flash[ALERT_MESSAGES] = Hash[]
		end

		def ensure_flash_message_type_exists(message_type)
			reset_flash_messages if @flash[ALERT_MESSAGES].nil?
			flash_messages[message_type] = [] unless flash_messages.key? message_type
		end

		# Accessing regular messages

		def messages()
			@messages
		end
	end
end